@@ -148,16 +148,18 @@ REST_API_VERSION_HISTORY = """
              ("GET /v3/{project_id}/volumes/detail") and volume-show
              ("GET /v3/{project_id}/volumes/{volume_id}") calls.
     * 3.64 - Include 'encryption_key_id' in volume and backup details
+    * 3.65 - Include 'consumes_quota' in volume and snapshot details
+           - Accept 'consumes_quota' filter in volume and snapshot list
+             operation.
+    * 3.66 - Allow snapshotting in-use volumes without force flag.
 """
 
 # The minimum and maximum versions of the API supported
 # The default api version request is defined to be the
 # minimum version of the API supported.
-# Explicitly using /v2 endpoints will still work
 _MIN_API_VERSION = "3.0"
-_MAX_API_VERSION = "3.64"
-_LEGACY_API_VERSION2 = "2.0"
-UPDATED = "2021-02-03T00:00:00Z"
+_MAX_API_VERSION = "3.66"
+UPDATED = "2021-09-16T00:00:00Z"
 
 
 # NOTE(cyeoh): min and max versions declared as functions so we can
@@ -171,10 +173,6 @@ def max_api_version():
     return APIVersionRequest(_MAX_API_VERSION)
 
 
-def legacy_api_version2():
-    return APIVersionRequest(_LEGACY_API_VERSION2)
-
-
 class APIVersionRequest(utils.ComparableMixin):
     """This class represents an API Version Request.
 
