@@ -59,11 +59,11 @@ IMPORT_VOLUME_ID = '00000000-0000-0000-0000-000000000000'
 class API(base.Base):
     """API for interacting with the volume backup manager."""
 
-    def __init__(self, db=None):
+    def __init__(self):
         self.backup_rpcapi = backup_rpcapi.BackupAPI()
         self.scheduler_rpcapi = scheduler_rpcapi.SchedulerAPI()
         self.volume_api = cinder.volume.API()
-        super(API, self).__init__(db)
+        super().__init__()
 
     def get(self, context, backup_id):
         backup = objects.Backup.get_by_id(context, backup_id)
@@ -499,23 +499,6 @@ class API(base.Base):
             msg = _('Provided backup record is missing size attribute')
             raise exception.InvalidInput(reason=msg)
 
-        # Try to get the backup with that ID in all projects even among
-        # deleted entries (we reuse soft-deleted backups).
-        try:
-            backup = objects.BackupImport.get_by_id(
-                context.elevated(read_deleted='yes'),
-                backup_record['id'],
-                project_only=False)
-
-            # If record exists and it's not deleted we cannot proceed
-            # with the import
-            if backup.status != fields.BackupStatus.DELETED:
-                msg = _('Backup already exists in database.')
-                raise exception.InvalidBackup(reason=msg)
-        except exception.BackupNotFound:
-            pass
-
-        # Check that we're under limit by reserving quota
         try:
             reserve_opts = {'backups': 1,
                             'backup_gigabytes': backup_record['size']}
@@ -537,16 +520,30 @@ class API(base.Base):
         }
 
         try:
-            if backup:
-                # "revive" the soft-deleted backup record retrieved earlier
+            try:
+                # Try to get the backup with that ID in all projects even among
+                # deleted entries.
+                backup = objects.BackupImport.get_by_id(
+                    context.elevated(read_deleted='yes'),
+                    backup_record['id'],
+                    project_only=False)
+
+                # If record exists and it's not deleted we cannot proceed
+                # with the import
+                if backup.status != fields.BackupStatus.DELETED:
+                    msg = _('Backup already exists in database.')
+                    raise exception.InvalidBackup(reason=msg)
+
+                # Otherwise we'll "revive" delete backup record
                 backup.update(kwargs)
                 backup.save()
-            else:
-                # create a new backup with the specified ID
+                QUOTAS.commit(context, reservations)
+            except exception.BackupNotFound:
+                # If record doesn't exist create it with the specific ID
                 backup = objects.BackupImport(context=context,
                                               id=backup_record['id'], **kwargs)
                 backup.create()
-            QUOTAS.commit(context, reservations)
+                QUOTAS.commit(context, reservations)
         except Exception:
             with excutils.save_and_reraise_exception():
                 try:
