@@ -15,7 +15,6 @@
 from oslo_config import cfg
 from oslo_serialization import base64
 from oslo_serialization import jsonutils
-from oslo_utils import versionutils
 from oslo_versionedobjects import fields
 
 from cinder import db
@@ -40,10 +39,12 @@ class Backup(base.CinderPersistentObject, base.CinderObject,
     # Version 1.4: Add restore_volume_id
     # Version 1.5: Add metadata
     # Version 1.6: Add encryption_key_id
-    VERSION = '1.6'
+    # Version 1.7: Add parent
+    VERSION = '1.7'
 
-    OPTIONAL_FIELDS = ('metadata',)
+    OPTIONAL_FIELDS = ('metadata', 'parent')
 
+    # NOTE: When adding a field obj_make_compatible needs to be updated
     fields = {
         'id': fields.UUIDField(),
 
@@ -55,6 +56,7 @@ class Backup(base.CinderPersistentObject, base.CinderObject,
         'availability_zone': fields.StringField(nullable=True),
         'container': fields.StringField(nullable=True),
         'parent_id': fields.StringField(nullable=True),
+        'parent': fields.ObjectField('Backup', nullable=True),
         'status': c_fields.BackupStatusField(nullable=True),
         'fail_reason': fields.StringField(nullable=True),
         'size': fields.IntegerField(nullable=True),
@@ -108,11 +110,6 @@ class Backup(base.CinderPersistentObject, base.CinderObject,
     def has_dependent_backups(self):
         return bool(self.num_dependent_backups)
 
-    def obj_make_compatible(self, primitive, target_version):
-        """Make an object representation compatible with a target version."""
-        super(Backup, self).obj_make_compatible(primitive, target_version)
-        target_version = versionutils.convert_version_to_tuple(target_version)
-
     @classmethod
     def _from_db_object(cls, context, backup, db_backup, expected_attrs=None):
         if expected_attrs is None:
@@ -148,6 +145,11 @@ class Backup(base.CinderPersistentObject, base.CinderObject,
         if not self._context:
             raise exception.OrphanedObjectError(method='obj_load_attr',
                                                 objtype=self.obj_name())
+        if attrname == 'parent':
+            if self.parent_id:
+                self.parent = self.get_by_id(self._context, self.parent_id)
+            else:
+                self.parent = None
         self.obj_reset_changes(fields=[attrname])
 
     def obj_what_changed(self):
@@ -174,6 +176,7 @@ class Backup(base.CinderPersistentObject, base.CinderObject,
                 self.metadata = db.backup_metadata_update(self._context,
                                                           self.id, metadata,
                                                           True)
+            updates.pop('parent', None)
             db.backup_update(self._context, self.id, updates)
 
         self.obj_reset_changes()
@@ -203,7 +206,7 @@ class Backup(base.CinderPersistentObject, base.CinderObject,
         # We don't want to export extra fields and we want to force lazy
         # loading, so we can't use dict(self) or self.obj_to_primitive
         record = {name: field.to_primitive(self, name, getattr(self, name))
-                  for name, field in self.fields.items()}
+                  for name, field in self.fields.items() if name != 'parent'}
         # We must update kwargs instead of record to ensure we don't overwrite
         # "real" data from the backup
         kwargs.update(record)
@@ -247,8 +250,10 @@ class BackupList(base.ObjectListBase, base.CinderObject):
                                   backups, expected_attrs=expected_attrs)
 
     @classmethod
-    def get_all_by_volume(cls, context, volume_id, filters=None):
-        backups = db.backup_get_all_by_volume(context, volume_id, filters)
+    def get_all_by_volume(
+            cls, context, volume_id, vol_project_id, filters=None):
+        backups = db.backup_get_all_by_volume(
+            context, volume_id, vol_project_id, filters)
         expected_attrs = Backup._get_expected_attrs(context)
         return base.obj_make_list(context, cls(context), objects.Backup,
                                   backups, expected_attrs=expected_attrs)
