@@ -39,15 +39,7 @@ CONF = cfg.CONF
 
 def app():
     # no auth, just let environ['cinder.context'] pass through
-    api = fakes.router.APIRouter()
-    mapper = fakes.urlmap.URLMap()
-    mapper['/v2'] = api
-    return mapper
-
-
-def app_v3():
-    # no auth, just let environ['cinder.context'] pass through
-    api = fakes.router.APIRouter()
+    api = fakes.router_v3.APIRouter()
     mapper = fakes.urlmap.URLMap()
     mapper['/v3'] = api
     return mapper
@@ -73,24 +65,9 @@ def service_get(context, service_id, backend_match_level=None, host=None,
 
 # Some of the tests check that volume types are correctly validated during a
 # volume manage operation.  This data structure represents an existing volume
-# type.  NOTE: cinder.db.sqlalchemy.volume_type_get() returns a dict describing
-# a specific volume type; this dict always contains an 'extra_specs' key.
-fake_vt = {
-    'id': fake.VOLUME_TYPE_ID,
-    'name': 'good_fakevt',
-    'extra_specs': {},
-}
-
-fake_encrypted_vt = {
-    'id': fake.VOLUME_TYPE2_ID,
-    'name': 'fake_encrypted_vt',
-    'extra_specs': {},
-    'encryption': {
-        'cipher': 'fake_cipher',
-        'control_location': 'front-end',
-        'key_size': 256,
-        'provider': 'fake_provider'},
-}
+# type.
+fake_vt = {'id': fake.VOLUME_TYPE_ID,
+           'name': 'good_fakevt'}
 
 
 def vt_get_volume_type_by_name(context, name):
@@ -102,8 +79,6 @@ def vt_get_volume_type_by_name(context, name):
     """
     if name == fake_vt['name']:
         return fake_vt
-    if name == fake_encrypted_vt['name']:
-        return fake_encrypted_vt
     raise exception.VolumeTypeNotFoundByName(volume_type_name=name)
 
 
@@ -116,37 +91,9 @@ def vt_get_volume_type(context, vt_id):
     """
     if vt_id == fake_vt['id']:
         return fake_vt
-    if vt_id == fake_encrypted_vt['id']:
-        return fake_encrypted_vt
     raise exception.VolumeTypeNotFound(volume_type_id=vt_id)
 
 
-def vt_get_default_volume_type(context):
-    """Replacement for cinder.volume.volume_types.get_default_volume_type.
-
-    If you want to use a specific fake volume type defined above, set
-    the flag for default_volume_type to the name of that fake type.
-
-    If you want to raise VolumeTypeDefaultMisconfiguredError, then set
-    the flag for default_volume_type to None.
-
-    Otherwise, for *any* non-None value of default_volume_type, this
-    will return our generic fake volume type.  (NOTE: by default,
-    CONF.default_volume_type is '__DEFAULT__'.)
-
-    """
-    default_vt_name = CONF.default_volume_type
-    if not default_vt_name:
-        raise exception.VolumeTypeDefaultMisconfiguredError(
-            volume_type_name='from vt_get_default_volume_type')
-    try:
-        default_vt = vt_get_volume_type_by_name(context, default_vt_name)
-    except exception.VolumeTypeNotFoundByName:
-        default_vt = fake_vt
-
-    return default_vt
-
-
 def api_manage(*args, **kwargs):
     """Replacement for cinder.volume.api.API.manage_existing.
 
@@ -198,8 +145,6 @@ def api_get_manageable_volumes(*args, **kwargs):
 
 @ddt.ddt
 @mock.patch('cinder.db.sqlalchemy.api.service_get', service_get)
-@mock.patch('cinder.volume.volume_types.get_default_volume_type',
-            vt_get_default_volume_type)
 @mock.patch('cinder.volume.volume_types.get_volume_type_by_name',
             vt_get_volume_type_by_name)
 @mock.patch('cinder.volume.volume_types.get_volume_type',
@@ -228,17 +173,7 @@ class VolumeManageTest(test.TestCase):
                                                       is_admin=False)
         self.controller = volume_manage.VolumeManageController()
 
-    def _get_resp_post(self, body):
-        """Helper to execute a POST os-volume-manage API call."""
-        req = webob.Request.blank('/v2/%s/os-volume-manage' % fake.PROJECT_ID)
-        req.method = 'POST'
-        req.headers['Content-Type'] = 'application/json'
-        req.environ['cinder.context'] = self._admin_ctxt
-        req.body = jsonutils.dump_as_bytes(body)
-        res = req.get_response(app())
-        return res
-
-    def _get_resp_post_v3(self, body, version):
+    def _get_resp_post(self, body, version='3.11'):
         """Helper to execute a POST os-volume-manage API call."""
         req = webob.Request.blank('/v3/%s/os-volume-manage' % fake.PROJECT_ID)
         req.method = 'POST'
@@ -247,7 +182,7 @@ class VolumeManageTest(test.TestCase):
         req.headers["OpenStack-API-Version"] = "volume " + version
         req.api_version_request = api_version.APIVersionRequest(version)
         req.body = jsonutils.dump_as_bytes(body)
-        res = req.get_response(app_v3())
+        res = req.get_response(app())
         return res
 
     @ddt.data({'host': 'host_ok'},
@@ -428,7 +363,7 @@ class VolumeManageTest(test.TestCase):
         detail = ""
         if detailed:
             detail = "/detail"
-        url = "/v2/%s/os-volume-manage%s%s" % (fake.PROJECT_ID, detail,
+        url = "/v3/%s/os-volume-manage%s%s" % (fake.PROJECT_ID, detail,
                                                query_string)
         req = webob.Request.blank(url)
         req.method = 'GET'
@@ -533,147 +468,12 @@ class VolumeManageTest(test.TestCase):
         self.assertTrue(mock_is_up.called)
 
     @mock.patch('cinder.volume.api.API.manage_existing', wraps=api_manage_new)
-    def test_manage_volume_with_creating_status_in_v3(self, mock_api_manage):
+    def test_manage_volume_with_creating_status(self, mock_api_manage):
         """Test managing volume to return 'creating' status in V3 API."""
         body = {'volume': {'host': 'host_ok',
                            'ref': 'fake_ref'}}
-        res = self._get_resp_post_v3(body, mv.ETAGS)
+        res = self._get_resp_post(body, mv.ETAGS)
         self.assertEqual(HTTPStatus.ACCEPTED, res.status_int)
         self.assertEqual(1, mock_api_manage.call_count)
         self.assertEqual('creating',
                          jsonutils.loads(res.body)['volume']['status'])
-
-    @mock.patch('cinder.volume.api.API.manage_existing', wraps=api_manage_new)
-    def test_manage_volume_with_creating_status_in_v2(self, mock_api_manage):
-        """Test managing volume to return 'creating' status in V2 API."""
-
-        body = {'volume': {'host': 'host_ok',
-                           'ref': 'fake_ref'}}
-        res = self._get_resp_post(body)
-        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int)
-        self.assertEqual(1, mock_api_manage.call_count)
-        self.assertEqual('creating',
-                         jsonutils.loads(res.body)['volume']['status'])
-
-    @ddt.data({'volume': {}}, None)
-    def test_manage_volume_with_invalid_body(self, body):
-        res = self._get_resp_post(body)
-        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
-
-    def test_negative_manage_to_encrypted_type(self):
-        """Not allowed to manage a volume to an encrypted volume type."""
-        ctxt = context.RequestContext(fake.USER_ID, fake.PROJECT_ID,
-                                      is_admin=True)
-        body = {'volume': {'host': 'host_ok',
-                           'ref': 'fake_ref',
-                           'volume_type': fake_encrypted_vt['name']}}
-        req = webob.Request.blank('/v3/%s/os-volume-manage' % fake.PROJECT_ID)
-        req.method = 'POST'
-        req.headers['Content-Type'] = 'application/json'
-        req.body = jsonutils.dump_as_bytes(body)
-        res = req.get_response(fakes.wsgi_app(fake_auth_context=ctxt))
-        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
-
-    def test_negative_manage_to_encrypted_default_type(self):
-        """Fail if no vol type in request and default vol type is encrypted."""
-
-        self.flags(default_volume_type=fake_encrypted_vt['name'])
-        ctxt = context.RequestContext(fake.USER_ID, fake.PROJECT_ID,
-                                      is_admin=True)
-        body = {'volume': {'host': 'host_ok',
-                           'ref': 'fake_ref'}}
-        req = webob.Request.blank('/v3/%s/os-volume-manage' % fake.PROJECT_ID)
-        req.method = 'POST'
-        req.headers['Content-Type'] = 'application/json'
-        req.body = jsonutils.dump_as_bytes(body)
-        res = req.get_response(fakes.wsgi_app(fake_auth_context=ctxt))
-        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
-
-    def test_negative_no_volume_type(self):
-        """Fail when no volume type is available for the managed volume."""
-        self.flags(default_volume_type=None)
-        ctxt = context.RequestContext(fake.USER_ID, fake.PROJECT_ID,
-                                      is_admin=True)
-        body = {'volume': {'host': 'host_ok',
-                           'ref': 'fake_ref'}}
-        req = webob.Request.blank('/v3/%s/os-volume-manage' % fake.PROJECT_ID)
-        req.method = 'POST'
-        req.headers['Content-Type'] = 'application/json'
-        req.body = jsonutils.dump_as_bytes(body)
-        res = req.get_response(fakes.wsgi_app(fake_auth_context=ctxt))
-        self.assertEqual(HTTPStatus.INTERNAL_SERVER_ERROR, res.status_int)
-
-    @mock.patch('cinder.group.API')
-    @mock.patch('cinder.flow_utils')
-    @mock.patch('cinder.volume.flows.api.manage_existing.get_flow')
-    @mock.patch('cinder.volume.api.API._get_service_by_host_cluster')
-    def test_manage_when_default_type_is_encrypted(self,
-                                                   mock_get_cluster,
-                                                   mock_get_flow,
-                                                   mock_flow_utils,
-                                                   mock_group_api):
-        """Default type doesn't matter if non-encrypted type is in request."""
-
-        # make an encrypted type the default volume type
-        self.flags(default_volume_type=fake_encrypted_vt['name'])
-        ctxt = context.RequestContext(fake.USER_ID, fake.PROJECT_ID,
-                                      is_admin=True)
-
-        # pass a non-encrypted volume type in the request
-        requested_vt = fake_vt
-        body = {'volume': {'host': 'host_ok',
-                           'ref': 'fake_ref',
-                           'volume_type': requested_vt['name']}}
-        req = webob.Request.blank('/v3/%s/os-volume-manage' % fake.PROJECT_ID)
-        req.method = 'POST'
-        req.headers['Content-Type'] = 'application/json'
-        req.body = jsonutils.dump_as_bytes(body)
-        res = req.get_response(fakes.wsgi_app(fake_auth_context=ctxt))
-
-        # request should be accepted
-        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int)
-
-        # make sure the volume type passed through is the specified one
-        args, _ = mock_get_flow.call_args
-        called_with = args[2]
-        self.assertEqual(requested_vt['name'],
-                         called_with['volume_type']['name'])
-        self.assertEqual(requested_vt['id'],
-                         called_with['volume_type']['id'])
-
-    @mock.patch('cinder.group.API')
-    @mock.patch('cinder.flow_utils')
-    @mock.patch('cinder.volume.flows.api.manage_existing.get_flow')
-    @mock.patch('cinder.volume.api.API._get_service_by_host_cluster')
-    def test_manage_with_default_type(self,
-                                      mock_get_cluster,
-                                      mock_get_flow,
-                                      mock_flow_utils,
-                                      mock_group_api):
-        """A non-encrypted default volume type should cause no problems."""
-
-        # make an non-encrypted type the default volume type
-        default_vt = fake_vt
-        self.flags(default_volume_type=default_vt['name'])
-        ctxt = context.RequestContext(fake.USER_ID, fake.PROJECT_ID,
-                                      is_admin=True)
-
-        # don't pass a volume type in the request
-        body = {'volume': {'host': 'host_ok',
-                           'ref': 'fake_ref'}}
-        req = webob.Request.blank('/v3/%s/os-volume-manage' % fake.PROJECT_ID)
-        req.method = 'POST'
-        req.headers['Content-Type'] = 'application/json'
-        req.body = jsonutils.dump_as_bytes(body)
-        res = req.get_response(fakes.wsgi_app(fake_auth_context=ctxt))
-
-        # request should be accepted
-        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int)
-
-        # make sure the volume type passed through is the default
-        args, _ = mock_get_flow.call_args
-        called_with = args[2]
-        self.assertEqual(default_vt['name'],
-                         called_with['volume_type']['name'])
-        self.assertEqual(default_vt['id'],
-                         called_with['volume_type']['id'])
