@@ -487,12 +487,13 @@ class GeneralFiltersTest(test.TestCase):
                'expected': ["name", "status", "metadata",
                             "bootable", "migration_status",
                             "availability_zone", "group_id",
-                            "size", "created_at", "updated_at"]},
+                            "size", "created_at", "updated_at",
+                            "consumes_quota"]},
               {'resource': 'backup',
                'expected': ["name", "status", "volume_id"]},
               {'resource': 'snapshot',
                'expected': ["name", "status", "volume_id", "metadata",
-                            "availability_zone"]},
+                            "availability_zone", "consumes_quota"]},
               {'resource': 'group_snapshot',
                'expected': ["name", "status", "group_id"]},
               {'resource': 'attachment',
