@@ -17,12 +17,13 @@ from oslo_utils import importutils
 
 from cinder import context
 from cinder import db
+from cinder.db.sqlalchemy import api as sqla_db
 from cinder import exception
 from cinder.objects import fields
 from cinder.objects import volume_attachment
-from cinder import test
 from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 from cinder.tests.unit import utils as tests_utils
 from cinder.volume import configuration as conf
 
@@ -48,7 +49,8 @@ class AttachmentManagerTestCase(test.TestCase):
     @mock.patch.object(db.sqlalchemy.api, '_volume_type_get',
                        v2_fakes.fake_volume_type_get)
     @mock.patch('cinder.db.sqlalchemy.api.volume_type_qos_specs_get')
-    def test_attachment_update(self, mock_type_get):
+    @mock.patch('cinder.volume.volume_types.get_volume_type_extra_specs')
+    def test_attachment_update(self, get_extra_specs, mock_type_get):
         """Test attachment_update."""
         volume_params = {'status': 'available'}
         connector = {
@@ -74,10 +76,12 @@ class AttachmentManagerTestCase(test.TestCase):
             expected = {
                 'encrypted': False,
                 'qos_specs': None,
+                'cacheable': False,
                 'access_mode': 'rw',
                 'driver_volume_type': 'iscsi',
                 'attachment_id': attachment_ref.id}
 
+            get_extra_specs.return_value = ''
             self.assertEqual(expected,
                              self.manager.attachment_update(
                                  self.context,
@@ -89,6 +93,21 @@ class AttachmentManagerTestCase(test.TestCase):
                                                 attachment_ref.instance_uuid,
                                                 connector['host'],
                                                 "na")
+            expected = {
+                'encrypted': False,
+                'qos_specs': None,
+                'cacheable': True,
+                'access_mode': 'rw',
+                'driver_volume_type': 'iscsi',
+                'attachment_id': attachment_ref.id}
+
+            get_extra_specs.return_value = '<is> True'
+            self.assertEqual(expected,
+                             self.manager.attachment_update(
+                                 self.context,
+                                 vref,
+                                 connector,
+                                 attachment_ref.id))
 
             new_attachment_ref = db.volume_attachment_get(self.context,
                                                           attachment_ref.id)
@@ -135,14 +154,16 @@ class AttachmentManagerTestCase(test.TestCase):
         attachment1.id = fake.UUID1
         attachment2.id = fake.UUID2
 
-        @mock.patch.object(self.manager.db, 'volume_admin_metadata_delete')
-        @mock.patch.object(self.manager.db, 'volume_detached')
+        @mock.patch('cinder.objects.VolumeAttachment.get_by_id',
+                    side_effect=[attachment1, attachment2])
+        @mock.patch.object(sqla_db, 'volume_admin_metadata_delete')
+        @mock.patch.object(sqla_db, 'volume_detached')
         @mock.patch.object(self.context, 'elevated')
         @mock.patch.object(self.manager, '_connection_terminate')
         @mock.patch.object(self.manager.driver, 'remove_export')
         @mock.patch.object(self.manager.driver, 'detach_volume')
-        def _test(mock_detach, mock_rm_export, mock_con_term,
-                  mock_elevated, mock_db_detached, mock_db_meta_delete):
+        def _test(mock_detach, mock_rm_export, mock_con_term, mock_elevated,
+                  mock_db_detached, mock_db_meta_delete, mock_get_attachment):
             mock_elevated.return_value = self.context
             mock_con_term.return_value = False
 
@@ -150,7 +171,7 @@ class AttachmentManagerTestCase(test.TestCase):
             # detach and remove_export
             vref.volume_attachment.objects.append(attachment1)
 
-            self.manager._do_attachment_delete(self.context, vref, attachment1)
+            self.manager.attachment_delete(self.context, attachment1.id, vref)
 
             mock_detach.assert_called_once_with(self.context, vref,
                                                 attachment1)
@@ -170,7 +191,7 @@ class AttachmentManagerTestCase(test.TestCase):
             mock_db_detached.reset_mock()
             mock_db_meta_delete.reset_mock()
 
-            self.manager._do_attachment_delete(self.context, vref, attachment2)
+            self.manager.attachment_delete(self.context, attachment2.id, vref)
 
             mock_rm_export.assert_not_called()
             mock_db_detached.called_once_with(self.context, vref,
@@ -207,25 +228,28 @@ class AttachmentManagerTestCase(test.TestCase):
                 self.assertFalse(has_shared_connection)
                 term_conn.assert_called_once_with(volume, {}, force=True)
 
+    @mock.patch('cinder.objects.VolumeAttachment.get_by_id')
     @mock.patch('cinder.volume.manager.VolumeManager.'
                 '_notify_about_volume_usage')
     @mock.patch('cinder.volume.manager.VolumeManager._connection_terminate',
                 return_value=None)
     @mock.patch('cinder.db.volume_detached')
     @mock.patch('cinder.db.volume_admin_metadata_delete')
-    def test_do_attachment_delete_none_shared_connection(self, mock_meta_del,
-                                                         mock_vol_detached,
-                                                         mock_conn_term,
-                                                         mock_notify):
-        # Tests that _do_attachment_delete does not call remove_export
+    def test_attachment_delete_none_shared_connection(self, mock_meta_del,
+                                                      mock_vol_detached,
+                                                      mock_conn_term,
+                                                      mock_notify,
+                                                      mock_get_attachment):
+        # Tests that attachment_delete does not call remove_export
         # if _connection_terminate returns None indicating there is nothing
         # to consider for the export.
         volume = mock.MagicMock()
-        attachment = mock.MagicMock()
         with mock.patch.object(self.manager.driver, '_initialized',
                                create=True, new=True):
             with mock.patch.object(self.manager.driver,
                                    'remove_export') as remove_export:
-                self.manager._do_attachment_delete(
-                    self.context, volume, attachment)
+                self.manager.attachment_delete(
+                    self.context, mock.sentinel.attachment_id, volume)
+                mock_get_attachment.assert_called_once_with(
+                    self.context, mock.sentinel.attachment_id)
                 remove_export.assert_not_called()
