@@ -61,6 +61,7 @@ from cinder.volume import volume_utils
 CONF = config.CONF
 
 _DB_CACHE = None
+DB_SCHEMA = None
 SESSION_CONFIGURED = False
 
 
@@ -70,32 +71,41 @@ class TestingException(Exception):
 
 class Database(fixtures.Fixture):
 
-    def __init__(self, db_api, db_migrate, sql_connection):
+    def __init__(self):
+        super().__init__()
+
         # NOTE(lhx_): oslo_db.enginefacade is configured in tests the same
         # way as it's done for any other services that uses the db
         global SESSION_CONFIGURED
         if not SESSION_CONFIGURED:
             sqla_api.configure(CONF)
             SESSION_CONFIGURED = True
-        self.sql_connection = sql_connection
 
         # Suppress logging for test runs
         migrate_logger = logging.getLogger('migrate')
         migrate_logger.setLevel(logging.WARNING)
 
-        self.engine = db_api.get_engine()
-        self.engine.dispose()
-        conn = self.engine.connect()
-        db_migrate.db_sync()
-        self._DB = "".join(line for line in conn.connection.iterdump())
-        self.engine.dispose()
-
     def setUp(self):
-        super(Database, self).setUp()
+        super().setUp()
+        engine = sqla_api.get_engine()
+        engine.dispose()
+        self._cache_schema()
+        conn = engine.connect()
+        conn.connection.executescript(DB_SCHEMA)
+        self.addCleanup(self.cleanup)
 
-        conn = self.engine.connect()
-        conn.connection.executescript(self._DB)
-        self.addCleanup(self.engine.dispose)
+    def _cache_schema(self):
+        global DB_SCHEMA
+        if not DB_SCHEMA:
+            engine = sqla_api.get_engine()
+            conn = engine.connect()
+            migration.db_sync()
+            DB_SCHEMA = "".join(line for line in conn.connection.iterdump())
+            engine.dispose()
+
+    def cleanup(self):
+        engine = sqla_api.get_engine()
+        engine.dispose()
 
 
 class TestCase(testtools.TestCase):
@@ -112,8 +122,10 @@ class TestCase(testtools.TestCase):
                                         RESOURCE_FILTER_FILENAME)
     MOCK_WORKER = True
     MOCK_TOOZ = True
+    FAKE_OVO_HISTORY_VERSION = '9999.999'
 
     def __init__(self, *args, **kwargs):
+
         super(TestCase, self).__init__(*args, **kwargs)
 
         # Suppress some log messages during test runs
@@ -229,11 +241,7 @@ class TestCase(testtools.TestCase):
         CONF.set_default('connection', 'sqlite://', 'database')
         CONF.set_default('sqlite_synchronous', False, 'database')
 
-        global _DB_CACHE
-        if not _DB_CACHE:
-            _DB_CACHE = Database(sqla_api, migration,
-                                 sql_connection=CONF.database.connection)
-        self.useFixture(_DB_CACHE)
+        self.useFixture(Database())
 
         # NOTE(blk-u): WarningsFixture must be after the Database fixture
         # because sqlalchemy-migrate messes with the warnings filters.
@@ -301,6 +309,16 @@ class TestCase(testtools.TestCase):
 
         self.vt = volume_types.get_default_volume_type()
 
+        # Create fake RPC history if we don't have enough to do tests
+        obj_versions = objects_base.OBJ_VERSIONS
+        if len(obj_versions) == 1:
+            vol_vers = obj_versions.get_current_versions()['Volume'].split('.')
+            new_volume_version = '%s.%s' % (vol_vers[0], int(vol_vers[1]) + 1)
+            obj_versions.add(self.FAKE_OVO_HISTORY_VERSION,
+                             {'Volume': new_volume_version})
+
+        self.latest_ovo_version = obj_versions.get_current()
+
     def _restore_obj_registry(self):
         objects_base.CinderObjectRegistry._registry._obj_classes = \
             self._base_test_obj_backup
