@@ -93,10 +93,10 @@ def create_volume(ctxt,
     if id:
         with mock.patch('cinder.objects.Volume.obj_attr_is_set',
                         obj_attr_is_set(objects.Volume)):
-            volume = objects.Volume(ctxt, id=id, **vol)
+            volume = objects.Volume(context=ctxt, id=id, **vol)
             volume.create()
     else:
-        volume = objects.Volume(ctxt, **vol)
+        volume = objects.Volume(context=ctxt, **vol)
         volume.create()
 
     # If we get a TestCase instance we add cleanup
@@ -108,12 +108,6 @@ def create_volume(ctxt,
 def attach_volume(ctxt, volume_id, instance_uuid, attached_host,
                   mountpoint, mode='rw'):
 
-    if isinstance(volume_id, objects.Volume):
-        volume_ovo = volume_id
-        volume_id = volume_ovo.id
-    else:
-        volume_ovo = None
-
     now = timeutils.utcnow()
     values = {}
     values['volume_id'] = volume_id
@@ -121,17 +115,10 @@ def attach_volume(ctxt, volume_id, instance_uuid, attached_host,
     values['mountpoint'] = mountpoint
     values['attach_time'] = now
 
-    attachment = db.volume_attach(ctxt, values)
+    attachment = db.volume_attach(ctxt.elevated(), values)
     volume, updated_values = db.volume_attached(
-        ctxt, attachment['id'], instance_uuid,
+        ctxt.elevated(), attachment['id'], instance_uuid,
         attached_host, mountpoint, mode)
-
-    if volume_ovo:
-        cls = objects.Volume
-        expected_attrs = cls._get_expected_attrs(ctxt)
-        volume = cls._from_db_object(ctxt, cls(ctxt), volume,
-                                     expected_attrs=expected_attrs)
-
     return volume
 
 
@@ -502,7 +489,8 @@ def create_service(ctxt, values=None):
     values = values or {}
     v = default_service_values()
     v.update(values)
-    service = db.service_create(ctxt, v)
+    service = objects.Service(ctxt, **v)
+    service.create()
     # We need to read the contents from the DB if we have set updated_at
     # or created_at fields
     if 'updated_at' in values or 'created_at' in values:
