@@ -12,16 +12,14 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-"""
-INFINIDAT InfiniBox Volume Driver
-"""
+"""INFINIDAT InfiniBox Volume Driver."""
 
 from contextlib import contextmanager
 import functools
 import platform
 import socket
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import units
@@ -31,7 +29,6 @@ from cinder import exception
 from cinder.i18n import _
 from cinder import interface
 from cinder.objects import fields
-from cinder import utils
 from cinder import version
 from cinder.volume import configuration
 from cinder.volume.drivers.san import san
@@ -132,9 +129,15 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         self.configuration.append_config_values(infinidat_opts)
         self._lookup_service = fczm_utils.create_lookup_service()
 
-    @staticmethod
-    def get_driver_options():
-        return infinidat_opts
+    @classmethod
+    def get_driver_options(cls):
+        additional_opts = cls._get_oslo_driver_opts(
+            'san_ip', 'san_login', 'san_password', 'use_chap_auth',
+            'chap_username', 'chap_password', 'san_thin_provision',
+            'use_multipath_for_image_xfer', 'enforce_multipath_for_image_xfer',
+            'num_volume_device_scan_tries', 'volume_dd_blocksize',
+            'max_over_subscription_ratio')
+        return infinidat_opts + additional_opts
 
     def _setup_and_get_system_object(self, management_address, auth):
         system = infinisdk.InfiniBox(management_address, auth=auth)
@@ -565,8 +568,9 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
     def _connection_context(self, volume):
         use_multipath = self.configuration.use_multipath_for_image_xfer
         enforce_multipath = self.configuration.enforce_multipath_for_image_xfer
-        connector = utils.brick_get_connector_properties(use_multipath,
-                                                         enforce_multipath)
+        connector = volume_utils.brick_get_connector_properties(
+            use_multipath,
+            enforce_multipath)
         connection = self.initialize_connection(volume, connector)
         try:
             yield connection
@@ -578,7 +582,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         use_multipath = self.configuration.use_multipath_for_image_xfer
         device_scan_attempts = self.configuration.num_volume_device_scan_tries
         protocol = connection['driver_volume_type']
-        connector = utils.brick_get_connector(
+        connector = volume_utils.brick_get_connector(
             protocol,
             use_multipath=use_multipath,
             device_scan_attempts=device_scan_attempts,
