@@ -107,7 +107,6 @@ class LVMVolumeDriver(driver.VolumeDriver):
         self.target_driver = importutils.import_object(
             target_driver,
             configuration=self.configuration,
-            db=self.db,
             executor=self._execute)
         self.protocol = self.target_driver.protocol
         self._sparse_copy_volume = False
@@ -581,17 +580,6 @@ class LVMVolumeDriver(driver.VolumeDriver):
                     image_service):
         return None, False
 
-    def get_volume_stats(self, refresh=False):
-        """Get volume status.
-
-        If 'refresh' is True, run update the stats first.
-        """
-
-        if refresh:
-            self._update_volume_stats()
-
-        return self._stats
-
     def extend_volume(self, volume, new_size):
         """Extend an existing volume's size."""
         self.vg.extend_volume(volume['name'],
